import importlib
import pkgutil
import sqlite3
from pathlib import Path
from sqlite3 import Connection
from typing import List, Tuple

import typer
from rich import print

config_sqlite_file = Path.home() / ".yoda" / "yoda.sqlite3"


def get_db_connection() -> Connection:
    conn = sqlite3.connect(config_sqlite_file)
    return conn


def _discover_yoda_modules():
    plugins = list()
    for package in ["core", "plugins"]:
        plugins_pkg = importlib.import_module(f'yodapa.{package}')
        plugins_path = plugins_pkg.__path__

        for finder, name, ispkg in pkgutil.iter_modules(plugins_path):
            # print("finder", finder, "name", name, "ispkg", ispkg)
            try:
                module = importlib.import_module(f'yodapa.{package}.{name}')
                if hasattr(module, "app") and isinstance(module.app, typer.Typer):
                    plugin_app: typer.Typer = module.app
                    plugins.append((name, plugin_app))
            except Exception as e:
                typer.echo(f"Failed to load plugin {name}: {e}", err=True)
    return plugins


def _discover_plugins_from_config_dir():
    plugins = list()
    local_plugins_dir = Path.home() / ".yoda" / "plugins"
    if local_plugins_dir.exists() and local_plugins_dir.is_dir():
        for finder, name, ispkg in pkgutil.iter_modules([str(local_plugins_dir)]):
            # print("finder", finder, "name", name, "ispkg", ispkg)
            try:
                module_path = local_plugins_dir / f"{name}.py"
                if not module_path.exists():
                    typer.echo(f"Plugin module {name} does not exist at {module_path}", err=True)
                    continue

                # Load the module from the file path
                spec = importlib.util.spec_from_file_location(name, str(module_path))
                if spec is None:
                    typer.echo(f"Could not load spec for module {name}", err=True)
                    continue

                module = importlib.util.module_from_spec(spec)
                spec.loader.exec_module(module)

                if hasattr(module, "app") and isinstance(module.app, typer.Typer):
                    plugin_app: typer.Typer = module.app
                    plugins.append((name, plugin_app))
            except Exception as e:
                typer.echo(f"Failed to load local plugin {name}: {e}", err=True)
    return plugins


def discover_plugins() -> List[Tuple[str, typer.Typer]]:
    """Discover plugins in the 'plugins' directory and the local plugins directory."""

    plugins: List[Tuple[str, typer.Typer]] = list()

    # 1. Discover plugins within the 'plugins' directory
    plugins += _discover_yoda_modules()

    # 2. Discover plugins in the local plugins directory
    plugins += _discover_plugins_from_config_dir()

    # print("Plugins discovered:", plugins)

    return plugins


def load_plugins(app: typer.Typer, plugins: List[Tuple[str, typer.Typer]]):
    """Load the plugins into the yoda typer app."""
    for name, plugin in plugins:
        try:
            app.add_typer(plugin, name=name)
            # typer.echo(f"Loaded plugin: {plugin.name}")
        except Exception as e:
            typer.echo(f"Error loading plugin {name}: {e}", err=True)


def init_plugins(app: typer.Typer):
    """Get plugins from both the disk and sqlite and only show the ones that are enabled
    TODO: Currently a very inefficient implementation, optimize in the future
    """

    plugins = discover_plugins()

    enabled_plugins = get_enabled_plugins()

    # print("Enabled plugins:", enabled_plugins)

    load_plugins(app,
                 [(plugin_name, typer_app) for plugin_name, typer_app in plugins if plugin_name in enabled_plugins])


def get_plugin_list():
    """Get plugin list from sqlite3"""
    conn = None
    try:
        conn = get_db_connection()
        cursor = conn.cursor()
        cursor.execute("SELECT name, enabled FROM plugins")
        rows = cursor.fetchall()

        return rows

    except sqlite3.OperationalError as e:
        if "no such table" in str(e):
            print("[red]Yoda config not initialized. Use [white]`yoda init`[red] to initialize[/]")
        else:
            print(f"An error occurred: {e}")
        return list()
    finally:
        if conn:
            conn.close()


def get_enabled_plugins():
    """Get plugin list from sqlite3"""
    return set(name for name, enabled in get_plugin_list() if enabled)


def _refresh_plugins():
    conn = None
    try:
        conn = get_db_connection()
        cursor = conn.cursor()

        # delete all existing plugins
        cursor.execute("DELETE FROM plugins")

        plugins = discover_plugins()
        for plugin_name, _ in plugins:
            cursor.execute("INSERT INTO plugins VALUES (?, TRUE)", (plugin_name,))

        conn.commit()
        print(f"[green] Plugins refreshed.[/]")

    except sqlite3.OperationalError as e:
        if "no such table" in str(e):
            print("[red]Yoda config not initialized. Use [white]`yoda init`[red] to initialize[/]")
        else:
            print(f"An error occurred: {e}")
    finally:
        if conn:
            conn.close()



import sqlite3

import typer
from rich import print
from rich.table import Table

from yodapa.core.util import _refresh_plugins, get_plugin_list, get_db_connection

PROTECTED_PLUGINS = ["config", "plugin"]

app = typer.Typer(help="Commands to manage plugins")


############################################
############### Commands ###################
############################################

@app.command(name="list")
def list_plugins():
    """List all available plugins"""
    rows = get_plugin_list()

    if not rows:
        print("[yellow] No plugins found.[/]")
        return

    table = Table(title="Yoda Plugins")
    table.add_column("Name", style="cyan", no_wrap=True)
    table.add_column("Enabled")

    for key, value in rows:
        table.add_row(key, "[green]Yes[/]" if value == 1 else "[red]No[/]")

    print(table)


@app.command(name="enable")
def enable_plugin(name: str):
    """Enable a plugin"""
    conn = None
    try:
        conn = get_db_connection()
        cursor = conn.cursor()
        cursor.execute("SELECT name, enabled FROM plugins WHERE name = ?", (name,))
        rows = cursor.fetchone()

        if not rows:
            print(f"[red] Plugin {name} not found.[/]")
            return

        cursor.execute("UPDATE plugins SET enabled=TRUE WHERE name = ?", (name,))
        conn.commit()
        print(f"[green] Enabled plugin {name}.[/]")

    except sqlite3.OperationalError as e:
        if "no such table" in str(e):
            print("[red]Yoda config not initialized. Use [white]`yoda init`[red] to initialize[/]")
        else:
            print(f"An error occurred: {e}")
    finally:
        if conn:
            conn.close()


@app.command(name="disable")
def disable_plugin(name: str):
    """Disable a plugin"""
    if name in PROTECTED_PLUGINS:
        print(f"[italic]{name}[/] [red]is an internal plugin that cannot be disabled[/]")
        return

    conn = None
    try:
        conn = get_db_connection()
        cursor = conn.cursor()
        cursor.execute("SELECT name, enabled FROM plugins WHERE name = ?", (name,))
        rows = cursor.fetchone()

        if not rows:
            print(f"[red] Plugin {name} not found.[/]")
            return

        cursor.execute("UPDATE plugins SET enabled=FALSE WHERE name = ?", (name,))
        conn.commit()
        print(f"[green]Disabled plugin {name}.[/]")

    except sqlite3.OperationalError as e:
        if "no such table" in str(e):
            print("[red]Yoda config not initialized. Use [white]`yoda init`[red] to initialize[/]")
        else:
            print(f"An error occurred: {e}")
    finally:
        if conn:
            conn.close()


@app.command(name="refresh")
def refresh_plugins():
    """Re-discovers all available plugins and enables them"""
    _refresh_plugins()



import sqlite3
import urllib

import typer
from rich import print

from yodapa.core.util import get_db_connection

app = typer.Typer()


@app.command(name="shorten")
def shorten_url(name: str, url: str):
    """Store URL as a short name"""

    conn = None
    try:
        conn = get_db_connection()
        cursor = conn.cursor()
        cursor.execute("INSERT INTO url VALUES (?, ?)", (name, urllib.parse.quote(url)))

        conn.commit()
        print(f"[green] URL stored.[/]")
    except sqlite3.OperationalError as e:
        if "no such table" in str(e):
            print("[red]Yoda config not initialized. Use [white]`yoda init`[red] to initialize[/]")
        else:
            print(f"An error occurred: {e}")
    finally:
        if conn:
            conn.close()


@app.command(name="expand")
def expand_short_url(name: str):
    """Get expanded URL from short name"""

    conn = None
    try:
        conn = get_db_connection()
        cursor = conn.cursor()
        cursor.execute("SELECT url FROM url WHERE name = ?", (name,))
        row = cursor.fetchone()

        if row:
            value = urllib.parse.unquote(row[0])
            print(f"[blue]{name}[/]: [bold]{value}[/]")
        else:
            print(f"[red]URL shortname [white]'{name}'[red] not found.[/]")
    except sqlite3.OperationalError as e:
        if "no such table" in str(e):
            print("[red]Yoda config not initialized. Use [white]`yoda init`[red] to initialize[/]")
        else:
            print(f"An error occurred: {e}")
    finally:
        if conn:
            conn.close()



import typer

app = typer.Typer(help="""
    Bye plugin. Say goodbye.

    Example:

        $ yoda bye goodbye --name MP

        $ yoda bye goodbye
    """)


@app.command()
def goodbye(name: str = None):
    """Say goodbye."""
    name = name or "Padawan"
    typer.echo(f"Goodbye {name}!")



import sqlite3

import typer
from rich import print
from rich.table import Table

from yodapa.core.util import config_sqlite_file, _refresh_plugins, get_db_connection

app = typer.Typer(help="Config management")


@app.command(name="init")
def initialize_config():
    """Initializes a sqlite db that will store the config"""
    if config_sqlite_file.exists():
        delete_old = typer.confirm("This file already exists. Do you want to continue?")

        if not delete_old:
            return

        # delete old file
        config_sqlite_file.unlink()

    conn = get_db_connection()

    cursor = conn.cursor()
    # create tables
    cursor.execute("""
        CREATE TABLE IF NOT EXISTS configurations (
            key TEXT PRIMARY KEY,
            value TEXT NOT NULL
        )
    """)

    cursor.execute("""
        CREATE TABLE IF NOT EXISTS plugins (
            name TEXT PRIMARY KEY,
            enabled BOOLEAN NOT NULL
        )
    """)

    cursor.execute("""
        CREATE TABLE IF NOT EXISTS url (
            name TEXT PRIMARY KEY,
            url TEXT NOT NULL
        )
    """)

    conn.commit()
    conn.close()

    # in addition to creating new tables, we must also load the available plugins
    _refresh_plugins()


@app.command(name="set")
def set_config_value(key: str, value: str):
    """Set a value for a key in config"""
    conn = None
    try:
        conn = get_db_connection()
        cursor = conn.cursor()
        cursor.execute("""
                INSERT INTO configurations (key, value)
                VALUES (?, ?)
                ON CONFLICT(key) DO UPDATE SET value=excluded.value
            """, (key, value))
        conn.commit()
        print(f"[green]Configuration set:[/] {key} = {value}")
    except sqlite3.OperationalError as e:
        if "no such table" in str(e):
            print("[red]Yoda config not initialized. Use [white]`yoda init`[red] to initialize[/]")
        else:
            print(f"An error occurred: {e}")
    finally:
        if conn:
            conn.close()


@app.command(name="get")
def get_config_value(key: str):
    """Get value of a key set in yoda config"""
    conn = None
    try:
        conn = get_db_connection()
        cursor = conn.cursor()
        cursor.execute("SELECT value FROM configurations WHERE key = ?", (key,))
        row = cursor.fetchone()

        if row:
            value = row[0]
            print(f"[blue]{key}[/]: {value}")
        else:
            print(f"[red]Configuration key [white]'{key}'[red] not found.[/]")
    except sqlite3.OperationalError as e:
        if "no such table" in str(e):
            print("[red]Yoda config not initialized. Use [white]`yoda init`[red] to initialize[/]")
        else:
            print(f"An error occurred: {e}")
    finally:
        if conn:
            conn.close()


@app.command(name="list")
def list_configurations():
    """List all key-value pairs in the config"""
    conn = None
    try:
        conn = get_db_connection()
        cursor = conn.cursor()
        cursor.execute("SELECT key, value FROM configurations")
        rows = cursor.fetchall()

        if not rows:
            print("[yellow] No configurations found.[/]")
            return

        table = Table(title="Configurations")
        table.add_column("Key", style="cyan", no_wrap=True)
        table.add_column("Value", style="magenta")

        for key, value in rows:
            table.add_row(key, value)

        print(table)

    except sqlite3.OperationalError as e:
        if "no such table" in str(e):
            print("[red]Yoda config not initialized. Use [white]`yoda init`[red] to initialize[/]")
        else:
            print(f"An error occurred: {e}")
    finally:
        if conn:
            conn.close()



import typer

app = typer.Typer(help="""
    Hi plugin. Say hello.

    Example:

        $ yoda hi hello --name MP

        $ yoda hi hello
    """)


@app.command()
def hello(name: str = None):
    """Say hello."""
    name = name or "Padawan"
    typer.echo(f"Hello {name}!")



import typer

from yodapa.core.config import initialize_config
from yodapa.core.util import init_plugins

app = typer.Typer()
init_plugins(app)


@app.command()
def init():
    """Init yoda configurations"""
    initialize_config()


if __name__ == "__main__":
    app()



import ollama
import typer
from rich.console import Console

app = typer.Typer(help="AI command. Allows you to communicate with your local LLMs")


def _communicate_with_ollama(prompt: str):
    try:
        response = ollama.chat(
            model="codellama",
            messages=[{"role": "system", "content": "You should talk like Yoda from star wars"},
                      {"role": "user", "content": prompt}],
            stream=False,
        )
        # typer.echo(f"Received response from Ollama: {response['message']['content'].strip()}")
        return response['message']['content'].strip()
    except ollama.ResponseError as e:
        typer.echo(f"Error communicating with Ollama: {e}", err=True)
        typer.echo(f"Failed to communicate with Ollama: {e}", err=True)

        typer.echo("If you don't have ollama installed, you can install it by going through the instructions on "
                   "their website: https://ollama.com/ and installing the codellama model")
        raise


@app.command()
def chat(prompt: str):
    """
    Chat with the AI model based on the provided prompt.

    Args:
        prompt (str): The prompt to start the conversation.
    """
    # typer.echo(f"Starting chat with prompt: {prompt}")

    try:
        # Interact with the Ollama LLM
        response = _communicate_with_ollama(prompt)
        typer.echo(f"🤖 AI response:\n{response}")

    except Exception as e:
        typer.echo(f"Error chatting with AI: {e}", err=True)
        typer.echo(f"Failed to chat with AI: {e}", err=True)


@app.command()
def generate_command(plugin_name: str, prompt: str):
    """
    Generate code for a new plugin command based on the provided prompt.

    Args:
        prompt (str): The description or functionality of the desired command.
        plugin_name (str): The name for the new plugin.
    """
    typer.echo(f"Generating command for plugin: {plugin_name} with prompt: {prompt}")

    # Construct the prompt for the AI model
    ai_prompt = f"""
Generate a Python Typer app named "{plugin_name}" with multiple commands as required based on the following description:

{prompt}.
    """
    try:
        console = Console()
        with console.status("[bold green]Waiting for AI response..."):
            response = ollama.chat(
                model="codellama",
                messages=[{"role": "system", "content": """You are an expert python programmer. You must use all the python best practices to write the most efficient python code. Provide complete working code for all the subcommands. Provide full working code. If the plugin requires storage, use local storage like files or sqlite, whichever is easier to use.
    You need to generate a typer command line app. An example app can be found below:
    ```python
    import typer
    
    app = typer.Typer(help=\"\"\"
        Hi plugin. Say hello.
    
        Example:
    
            $ yoda hi hello --name MP
    
            $ yoda hi hello
        \"\"\")
    
    
    @app.command()
    def hello(name: str = None):
        \"\"\"Say hello.\"\"\"
        name = name or "Padawan"
        typer.echo(f"Hello {{name}}!")
    ```   
    
    You must only return the generated code for the plugin class. All the details for the plugin class should be added in the docstring.
    When the user provides a description for their requirement, you must use all the best practices required to implement what they need.
                """},
                          {"role": "user", "content": ai_prompt}],
                stream=False,
            )
            # typer.echo(f"Received response from Ollama: {response['message']['content'].strip()}")
            generated_code = response['message']['content'].strip()
        typer.echo(f"🤖 Generated code:\n{generated_code}")
    except ollama.ResponseError as e:
        typer.echo(f"Error communicating with Ollama: {e}", err=True)
        typer.echo(f"Failed to communicate with Ollama: {e}", err=True)

        typer.echo("If you don't have ollama installed, you can install it by going through the instructions on "
                   "their website: https://ollama.com/ and installing the codellama model")



import os
import string
from typing import List

import typer
from rich import print

app = typer.Typer()


@app.command()
def speedtest():
    """Test your internet connection"""
    os.system("speedtest-cli")


@app.command()
def coinflip():
    """Flip a coin!"""
    import random
    value = random.randint(0, 1)
    if value == 1:
        print("[green]Heads! :speaking_head:[/]")
    else:
        print("[magenta]Tails![/]")


@app.command()
def checksite(url: str):
    """Check if a site is up and running"""
    import urllib.request
    try:
        code = urllib.request.urlopen(url).getcode()
        if code != 200:
            print(f"[bold red] Site is DOWN![/] HTTP code received: {code}")
        else:
            print("[bold green] Site is UP![/]")
    except Exception as e:
        print(e)
        print("[bold red] Site is DOWN![/]")


@app.command()
def whois(domain: str):
    """Check whois info for a domain name"""
    import whois
    whois_data = whois.whois(domain)
    print(whois_data.text)


@app.command()
def generatepassword():
    """Generates a secure password"""
    import random
    length = random.randint(16, 25)
    characters = string.ascii_letters + string.digits + string.punctuation
    result: List[str] = list()
    for _ in range(length):
        result.append(random.choice(characters))
    print(''.join(result))
